@@ -42,8 +42,8 @@
 #include "gstinfo.h"
 #include "gstvalue.h"
 #include "gstbuffer.h"
-#include "gstquark.h"
 #include "gststructure.h"
+#include "gstidstr-private.h"
 
 #include <gobject/gvaluecollector.h>
 #include <string.h>
@@ -68,12 +68,12 @@ typedef struct
 {
   GType type;                   /* type the data is in */
 
+  GstIdStr name;
   const gchar *nick;            /* translated short description */
   const gchar *blurb;           /* translated long description  */
 
   GstTagMergeFunc merge_func;   /* functions to merge the values */
   GstTagFlag flag;              /* type of tag */
-  GQuark name_quark;            /* quark for the name */
 }
 GstTagInfo;
 
@@ -410,7 +410,9 @@ _priv_gst_tag_initialize (void)
   gst_tag_register_static (GST_TAG_PRIVATE_DATA, GST_TAG_FLAG_META,
       GST_TYPE_SAMPLE,
       _("private-data"), _("Private data"), gst_tag_merge_use_first);
-
+  gst_tag_register_static (GST_TAG_CONTAINER_SPECIFIC_TRACK_ID,
+      GST_TAG_FLAG_META, G_TYPE_STRING,
+      _("container-specific-track-id"), _("Container-specific Track ID"), NULL);
 }
 
 /**
@@ -455,8 +457,7 @@ gst_tag_merge_strings_with_comma (GValue * dest, const GValue * src)
   }
 
   g_value_init (dest, G_TYPE_STRING);
-  g_value_take_string (dest, str->str);
-  g_string_free (str, FALSE);
+  g_value_take_string (dest, g_string_free (str, FALSE));
 }
 
 static GstTagInfo *
@@ -549,10 +550,10 @@ gst_tag_register_static (const gchar * name, GstTagFlag flag, GType type,
     return;
   }
 
-  info = g_slice_new (GstTagInfo);
+  info = g_new0 (GstTagInfo, 1);
   info->flag = flag;
   info->type = type;
-  info->name_quark = g_quark_from_static_string (name);
+  gst_id_str_set_static_str (&info->name, name);
   info->nick = nick;
   info->blurb = blurb;
   info->merge_func = func;
@@ -693,7 +694,7 @@ gst_tag_list_new_internal (GstStructure * s, GstTagScope scope)
 
   g_assert (s != NULL);
 
-  tag_list = (GstTagList *) g_slice_new (GstTagListImpl);
+  tag_list = (GstTagList *) g_new (GstTagListImpl, 1);
 
   gst_mini_object_init (GST_MINI_OBJECT_CAST (tag_list), 0, GST_TYPE_TAG_LIST,
       (GstMiniObjectCopyFunction) __gst_tag_list_copy, NULL,
@@ -724,7 +725,7 @@ __gst_tag_list_free (GstTagList * list)
   memset (list, 0xff, sizeof (GstTagListImpl));
 #endif
 
-  g_slice_free1 (sizeof (GstTagListImpl), list);
+  g_free (list);
 }
 
 static GstTagList *
@@ -754,7 +755,7 @@ gst_tag_list_new_empty (void)
   GstStructure *s;
   GstTagList *tag_list;
 
-  s = gst_structure_new_id_empty (GST_QUARK (TAGLIST));
+  s = gst_structure_new_empty ("taglist");
   tag_list = gst_tag_list_new_internal (s, GST_TAG_SCOPE_STREAM);
   return tag_list;
 }
@@ -955,11 +956,11 @@ gst_tag_list_is_empty (const GstTagList * list)
 }
 
 static gboolean
-gst_tag_list_fields_equal (GQuark field_id, const GValue * value2,
+gst_tag_list_fields_equal (const GstIdStr * field, const GValue * value2,
     gpointer data)
 {
   const GstStructure *struct1 = (const GstStructure *) data;
-  const GValue *value1 = gst_structure_id_get_value (struct1, field_id);
+  const GValue *value1 = gst_structure_id_str_get_value (struct1, field);
   gdouble d1, d2;
 
   if (value1 == NULL) {
@@ -1016,7 +1017,8 @@ gst_tag_list_is_equal (const GstTagList * list1, const GstTagList * list2)
     return FALSE;
   }
 
-  return gst_structure_foreach (s1, gst_tag_list_fields_equal, (gpointer) s2);
+  return gst_structure_foreach_id_str (s1, gst_tag_list_fields_equal,
+      (gpointer) s2);
 }
 
 typedef struct
@@ -1032,7 +1034,7 @@ gst_tag_list_add_value_internal (GstTagList * tag_list, GstTagMergeMode mode,
 {
   GstStructure *list = GST_TAG_LIST_STRUCTURE (tag_list);
   const GValue *value2;
-  GQuark tag_quark;
+  const GstIdStr *tag_name;
 
   if (info == NULL) {
     info = gst_tag_lookup (tag);
@@ -1050,23 +1052,23 @@ gst_tag_list_add_value_internal (GstTagList * tag_list, GstTagMergeMode mode,
     return;
   }
 
-  tag_quark = info->name_quark;
+  tag_name = &info->name;
 
   if (info->merge_func
-      && (value2 = gst_structure_id_get_value (list, tag_quark)) != NULL) {
+      && (value2 = gst_structure_id_str_get_value (list, tag_name)) != NULL) {
     GValue dest = { 0, };
 
     switch (mode) {
       case GST_TAG_MERGE_REPLACE_ALL:
       case GST_TAG_MERGE_REPLACE:
-        gst_structure_id_set_value (list, tag_quark, value);
+        gst_structure_id_str_set_value (list, tag_name, value);
         break;
       case GST_TAG_MERGE_PREPEND:
         if (GST_VALUE_HOLDS_LIST (value2) && !GST_VALUE_HOLDS_LIST (value))
           gst_value_list_prepend_value ((GValue *) value2, value);
         else {
           gst_value_list_merge (&dest, value, value2);
-          gst_structure_id_take_value (list, tag_quark, &dest);
+          gst_structure_id_str_take_value (list, tag_name, &dest);
         }
         break;
       case GST_TAG_MERGE_APPEND:
@@ -1074,7 +1076,7 @@ gst_tag_list_add_value_internal (GstTagList * tag_list, GstTagMergeMode mode,
           gst_value_list_append_value ((GValue *) value2, value);
         else {
           gst_value_list_merge (&dest, value2, value);
-          gst_structure_id_take_value (list, tag_quark, &dest);
+          gst_structure_id_str_take_value (list, tag_name, &dest);
         }
         break;
       case GST_TAG_MERGE_KEEP:
@@ -1088,13 +1090,13 @@ gst_tag_list_add_value_internal (GstTagList * tag_list, GstTagMergeMode mode,
     switch (mode) {
       case GST_TAG_MERGE_APPEND:
       case GST_TAG_MERGE_KEEP:
-        if (gst_structure_id_get_value (list, tag_quark) != NULL)
+        if (gst_structure_id_str_get_value (list, tag_name) != NULL)
           break;
         /* fall through */
       case GST_TAG_MERGE_REPLACE_ALL:
       case GST_TAG_MERGE_REPLACE:
       case GST_TAG_MERGE_PREPEND:
-        gst_structure_id_set_value (list, tag_quark, value);
+        gst_structure_id_str_set_value (list, tag_name, value);
         break;
       case GST_TAG_MERGE_KEEP_ALL:
         break;
@@ -1106,13 +1108,13 @@ gst_tag_list_add_value_internal (GstTagList * tag_list, GstTagMergeMode mode,
 }
 
 static gboolean
-gst_tag_list_copy_foreach (GQuark tag_quark, const GValue * value,
+gst_tag_list_copy_foreach (const GstIdStr * field, const GValue * value,
     gpointer user_data)
 {
   GstTagCopyData *copy = (GstTagCopyData *) user_data;
   const gchar *tag;
 
-  tag = g_quark_to_string (tag_quark);
+  tag = gst_id_str_as_str (field);
   gst_tag_list_add_value_internal (copy->list, copy->mode, tag, value, NULL);
 
   return TRUE;
@@ -1142,7 +1144,7 @@ gst_tag_list_insert (GstTagList * into, const GstTagList * from,
   if (mode == GST_TAG_MERGE_REPLACE_ALL) {
     gst_structure_remove_all_fields (GST_TAG_LIST_STRUCTURE (into));
   }
-  gst_structure_foreach (GST_TAG_LIST_STRUCTURE (from),
+  gst_structure_foreach_id_str (GST_TAG_LIST_STRUCTURE (from),
       gst_tag_list_copy_foreach, &data);
 }
 
@@ -1399,20 +1401,20 @@ typedef struct
 TagForeachData;
 
 static int
-structure_foreach_wrapper (GQuark field_id, const GValue * value,
+structure_foreach_wrapper (const GstIdStr * field, const GValue * value,
     gpointer user_data)
 {
   TagForeachData *data = (TagForeachData *) user_data;
 
-  data->func (data->tag_list, g_quark_to_string (field_id), data->data);
+  data->func (data->tag_list, gst_id_str_as_str (field), data->data);
   return TRUE;
 }
 
 /**
  * gst_tag_list_foreach:
  * @list: list to iterate over
- * @func: (scope call): function to be called for each tag
- * @user_data: (closure): user specified data
+ * @func: (scope call) (closure user_data): function to be called for each tag
+ * @user_data: user specified data
  *
  * Calls the given function for each tag inside the tag list. Note that if there
  * is no tag, the function won't be called at all.
@@ -1429,7 +1431,7 @@ gst_tag_list_foreach (const GstTagList * list, GstTagForeachFunc func,
   data.func = func;
   data.tag_list = list;
   data.data = user_data;
-  gst_structure_foreach (GST_TAG_LIST_STRUCTURE (list),
+  gst_structure_foreach_id_str (GST_TAG_LIST_STRUCTURE (list),
       structure_foreach_wrapper, &data);
 }
 
